;
;
;
.zero
*     = 0
ptr           .dsb  2

mov_src       .dsb  2
mov_dst       .dsb  2
mov_siz       .dsb  2


.text
*     = START_ADDRESS



#define via_b         $0300
#define via_a         $0301
#define via_ddrb      $0302
#define via_ddra      $0303
#define via_t1cl      $0304
#define via_t1ch      $0305
#define via_t1ll      $0306
#define via_t1lh      $0307
#define via_t2ll      $0308
#define via_t2ch      $0309
#define via_sr        $030a
#define via_acr       $030b
#define via_pcr       $030c
#define via_ifr       $030d
#define via_ier       $030e
#define via_aor       $030f


#define stdcharset    $9900
#define textscreen    $bb80
#define hiresscreen   $a000



BIOS
; ==============================================================
; disable interrupts
      sei

      ldx #$ff              ; RESET
      txs                   ; Set stack pointer to #FF.
      cld

      lda #<irq_block       ; set interrupt hooks in RAM
      sta mov_src
      lda #>irq_block
      sta mov_src+1
      lda #<$244
      sta mov_dst
      lda #>$244
      sta mov_dst+1
      lda #<(irq_block_end-irq_block)
      sta mov_siz
      lda #>(irq_block_end-irq_block)
      sta mov_siz+1
      jsr mem_move
      
      jsr init_via
      jsr init_hires
      
      cli
      
; ==============================================================
; loop forever
halt  jmp halt


; ==============================================================
; default irq handler
def_irq
.(
      rti
.)
; ==============================================================
; default nmi handler
def_nmi
.(
      pha
      txa
      pha
      tya
      pha
      
      lda #$55
      cmp hiresscreen
      beq other
      bne fill
other
      lda #$6a
fill
      sta mov_src
      lda #<hiresscreen
      sta mov_dst
      lda #>hiresscreen
      sta mov_dst+1
      lda #<(40*200)
      sta mov_siz
      lda #>(40*200)
      sta mov_siz+1
      jsr mem_set

      pla
      tay
      pla
      tax
      pla
      rti
.)

; ==============================================================
; initialize hires mode
init_hires
.(
      lda #<defcharset
      sta mov_src
      lda #>defcharset
      sta mov_src+1
      lda #<stdcharset
      sta mov_dst
      lda #>stdcharset
      sta mov_dst+1
      lda #<defcharset_size
      sta mov_siz
      lda #>defcharset_size
      sta mov_siz+1
      jsr mem_move

      lda #$40
      sta mov_src
      lda #<hiresscreen
      sta mov_dst
      lda #>hiresscreen
      sta mov_dst+1
      lda #<(40*200)
      sta mov_siz
      lda #>(40*200)
      sta mov_siz+1
      jsr mem_set

      lda #28             ; Write text switch
      sta hiresscreen+(40*200)

      lda #30             ; Write hires switch
      sta $bfdf

      lda #$20
      sta mov_src
      lda #<textscreen+(40*25)
      sta mov_dst
      lda #>textscreen+(40*25)
      sta mov_dst+1
      lda #<(40*3)
      sta mov_siz
      lda #>(40*3)
      sta mov_siz+1
      jsr mem_set
      rts
.)

; ==============================================================
; init via defaults
init_via
.(
      lda #$ff
      sta via_ddra
      lda #$f7
      sta via_ddrb
      lda #$b7
      sta via_b
      lda #$dd
      sta via_pcr
      
      ; no via interrupts
      lda #$7f
      sta via_ier

      lda #$40
      sta via_acr

      lda #$00
      sta via_t1ll
      sta via_t1cl
      sta via_t2ll
      lda #$00
      sta via_t1lh
      sta via_t1ch
      sta via_t2ch

      ; disable mixer
      lda #7
      ldx #$3f
      jsr psgset
      
      ; mute all channels
      lda #8
      ldx #0
      jsr psgset
      lda #9
      ldx #0
      jsr psgset
      lda #10
      ldx #0
      jsr psgset
      
      lda #0
      ldx #$7f
      jsr psgset

      lda #1
      ldx #$00
      jsr psgset
      rts
.)

; ==============================================================
; write X to register A
psgset
.(
      sta     via_aor         ; Send A to port A of 6522.
      tay
      txa
      cpy     #$07            ; If writing to register 7, set
      bne     4+*             ; 1/0 port to output.
      ora     #$40

      tay
      lda     via_pcr         ; Set CA2 (BC1 of 8912) to 1,
      ora     #$ee            ; set CB2 (BDIR of 8912) to 1.
      sta     via_pcr         ; 8912 latches the address.
      and     #$11            ; Set CA2 and CB2 to 0, BC1 and
      ora     #$cc            ; BDIR in inactive state.
      sta     via_pcr         ;
      tax
      tya

      sta     via_aor         ; Send data to 8912 register.
      txa
      ora     #$ec            ; Set CA2 to 0 and CB2 to 1,
      sta     via_pcr         ; 8912 latches data.
      and     #$11            ; Set CA2 and CB2 to 0, BC1 and
      ora     #$cc            ; BDIR in inactive state.
      sta     via_pcr         ;
      rts
.)

;
; mem_move
;
mem_move
.(
          ldx #$00              ; this routine transfers a block
          ldy #$00              ; of data using mov_src as the
again     cpy mov_siz           ; source pointer and mov_dst as the
          bne next              ; destination pointer. the
          cpx mov_siz+1         ; length of data to be moved is
          beq exit              ; held in location mov_siz.
next      lda (mov_src),y
          sta (mov_dst),y
          iny
          bne again
          inc mov_src+1
          inc mov_dst+1
          inx
          jmp again
exit      rts
.)

;
; mem_set
;
mem_set
.(
          ldx #$00              ; this routine sets a block
          ldy #$00              ; of data using mov_src as the
again     cpy mov_siz           ; fill value and mov_dst as the
          bne next              ; destination pointer. the
          cpx mov_siz+1         ; length of data to be moved is
          beq exit              ; held in location mov_siz.
next      lda mov_src
          sta (mov_dst),y
          iny
          bne again
          inc mov_src+1
          inc mov_dst+1
          inx
          jmp again
exit      rts
.)

irq_block
          jmp   def_irq
          jmp   def_nmi
irq_block_end

#include "defcharset.s"
defcharset_end
defcharset_size = (defcharset_end-defcharset)

BIOS_END

    .dsb $FFFA - BIOS_END

_VECTORS

#if ( _VECTORS - $FFFA )
#error - Vector address is incorrect, diag will crash
#else

_VECTORNMI          .word $247              ; FFFA-FFFB - NMI Vector    (default: $0247)
_VECTORRESET        .word BIOS              ; FFFC-FFFD - RESET Vector  (default: $F88F)
_VECTORIRQ          .word $244              ; FFFE-FFFF - IRQ Vector    (default: $0244)

#echo Remaining space in the setup code:
#print (_VECTORS - BIOS_END)
#endif
